﻿using System.Collections.Generic;
using System.Linq;
using Telerik.Windows.Controls;
using Telerik.Windows.Persistence.Services;

namespace WpfApp52
{
    public class GridViewCustomPropertyProvider : ICustomPropertyProvider
    {
        public CustomPropertyInfo[] GetCustomProperties()
        {
            return new CustomPropertyInfo[]
            {
                new CustomPropertyInfo("Columns", typeof(List<ColumnProxy>)),
            };
        }

        public void InitializeObject(object context)
        {
            if (context is RadGridView)
            {
                RadGridView gridView = context as RadGridView;
                gridView.SortDescriptors.Clear();
                gridView.GroupDescriptors.Clear();
                gridView.Columns
                    .OfType<GridViewColumn>()
                    .Where(c => c.ColumnFilterDescriptor.IsActive)
                    .ToList().ForEach(c => c.ClearFilters());
            }
        }

        public object InitializeValue(CustomPropertyInfo customPropertyInfo, object context)
        {
            return null;
        }

        public object ProvideValue(CustomPropertyInfo customPropertyInfo, object context)
        {
            RadGridView gridView = context as RadGridView;

            if (customPropertyInfo.Name == "Columns")
            {
                List<ColumnProxy> columnProxies = new List<ColumnProxy>();

                foreach (GridViewColumn column in gridView.Columns)
                {
                    var gridName = column.DataControl.Name;
                    var columnHeader = column.Header.ToString();

                    if (column.DataControl.Name == "gridView")
                    {
                        if (columnHeader == "Info")
                        {
                            continue;
                        }
                    }
                    else if (column.DataControl.Name == "gridView2")
                    {
                        if (columnHeader == "Data")
                        {
                            continue;
                        }
                    }
                    // and so on

                    columnProxies.Add(new ColumnProxy()
                    {
                        UniqueName = column.UniqueName,
                        Header = column.Header.ToString(),
                        DisplayOrder = column.DisplayIndex,
                        Width = column.Width,
                    });
                }
                return columnProxies;
            }
            return null;
        }

        public void RestoreValue(CustomPropertyInfo customPropertyInfo, object context, object value)
        {
            RadGridView gridView = context as RadGridView;

            if (customPropertyInfo.Name == "Columns")
            {
                List<ColumnProxy> columnProxies = value as List<ColumnProxy>;

                foreach (ColumnProxy proxy in columnProxies.OrderBy(x => x.DisplayOrder))
                {
                    GridViewColumn column = gridView.Columns[proxy.UniqueName];
                    column.DisplayIndex = proxy.DisplayOrder;
                    column.Header = proxy.Header;
                    column.Width = proxy.Width;
                }
            }
        }
    }
}
